package org.RI.P1;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Scanner;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Comparator;

import org.apache.tika.Tika;
import org.apache.tika.exception.TikaException;
import org.apache.tika.language.detect.LanguageDetector;
import org.apache.tika.langdetect.OptimaizeLangDetector;
import org.apache.tika.language.detect.LanguageResult;
import org.apache.tika.metadata.*;
import org.apache.tika.parser.*;
import org.apache.tika.sax.BodyContentHandler;
import org.apache.tika.sax.LinkContentHandler;
import org.apache.tika.sax.TeeContentHandler;
import org.apache.tika.sax.ToHTMLContentHandler;
import org.apache.tools.ant.FileScanner;
import org.apache.tika.parser.html.HtmlParser;
import org.xml.sax.SAXException;

public class FileData {
    private File file;
    private String filename;
    private String type;
    private String encoding;
    private LanguageResult language;
    private InputStream inputStream;
    private Metadata metadata;
    private BodyContentHandler contentHandler;
    private ParseContext parseContext;
    private AutoDetectParser parser;
    private LanguageDetector langIdentifier;
    private LinkContentHandler linkHandler;
    private TeeContentHandler teeHandler;
    private ToHTMLContentHandler toHTMLhandler;
    private HashMap<String, Integer> wordFrequency;
    private ArrayList<Map.Entry<Integer, String>> orderedFrequencies;
    private Tika tika;

    FileData() {
    }

    FileData(File file) throws IOException, TikaException, SAXException {
        this.file = file;
        inputStream = new FileInputStream(file);
        metadata = new Metadata();
        parser = new AutoDetectParser();
        contentHandler = new BodyContentHandler(-1);
        parseContext = new ParseContext();
        langIdentifier = new OptimaizeLangDetector().loadModels();
        linkHandler = new LinkContentHandler();
        toHTMLhandler = new ToHTMLContentHandler();
        wordFrequency = new HashMap<>();
        orderedFrequencies = new ArrayList<>();
        tika = new Tika();
        setMetadata();
    }

    private void setMetadata() throws IOException, TikaException, SAXException {
        parser.parse(inputStream, contentHandler, metadata, parseContext);
        filename = metadata.get(TikaCoreProperties.TITLE);
        type = metadata.get(Metadata.CONTENT_TYPE);
        encoding = metadata.get(Metadata.CONTENT_ENCODING);
        language = langIdentifier.detect(contentHandler.toString());
    }

    void getLinks() throws IOException, TikaException, SAXException {
        HtmlParser htmlParser = new HtmlParser();
        inputStream = new FileInputStream(file);
        teeHandler = new TeeContentHandler(linkHandler, contentHandler, toHTMLhandler);
        htmlParser.parse(inputStream, teeHandler, metadata, parseContext);
        System.out.println("Filename: " + filename);
        System.out.println("Links: " + linkHandler.getLinks());
    }

    private void tokenizeFile() throws FileNotFoundException, IOException, TikaException {
        String fileContent = tika.parseToString(file);
        fileContent.toLowerCase();
        Scanner fileScanner = new Scanner(fileContent);
        Integer defaultValue = 0;
        Integer ocurrence;
        String delimiters = "\\s+|[\\.\\;\\:\\,]\\s+|[()¿?¡!]";
        fileScanner.useDelimiter(delimiters);

        while (fileScanner.hasNext()) {
            ocurrence = wordFrequency.getOrDefault(fileScanner, defaultValue);
            if (ocurrence == defaultValue) {
                wordFrequency.put(fileScanner.next(), defaultValue + 1);
            } else {
                wordFrequency.put(fileScanner.next(), ocurrence += 1);
            }
        }
    }

    private void sortWords() {
        for (Entry<String, Integer> item : wordFrequency.entrySet()) {
            orderedFrequencies.add(Map.entry(item.getValue(), item.getKey()));
        }

        Collections.sort(orderedFrequencies, new Comparator<Map.Entry<Integer, String>>() {
            @Override
            public int compare(Map.Entry<Integer, String> m1, Map.Entry<Integer, String> m2) {
                return (m1.getValue()).compareTo(m2.getValue());
            }
        });
    }

    private void saveFrequency() throws FileNotFoundException {
        FileOutputStream outputFile = new FileOutputStream("output/output_" + filename + ".dat");
        PrintStream output = new PrintStream(outputFile);
        System.setOut(output);

        for (Entry<Integer, String> item : orderedFrequencies) {
            System.out.println(item.getValue() + " " + item.getKey());
        }

    }

    void setWordOcurrences() throws FileNotFoundException, IOException, TikaException {
        tokenizeFile();
        sortWords();
        saveFrequency();
    }

    @Override
    public String toString() {
        return "Filename: " + filename + "\n" + "Type: " + type + "\n" + "Encoding: " + encoding + "\n" + "Language: "
                + language.getLanguage() + "\n";
    }

}
